@@ -5,6 +5,7 @@ import logging
 import mimetypes
 import os
 from typing import Any
+from typing import Dict
 from typing import Optional
 from typing import Tuple
 from typing import Union
@@ -28,6 +29,7 @@ else:
 
 API_ROOT = 'https://www.googleapis.com/storage/v1/b'
 API_ROOT_UPLOAD = 'https://www.googleapis.com/upload/storage/v1/b'
+VERIFY_SSL = True
 SCOPES = [
     'https://www.googleapis.com/auth/devstorage.read_write',
 ]
@@ -35,6 +37,13 @@ SCOPES = [
 MAX_CONTENT_LENGTH_SIMPLE_UPLOAD = 5 * 1024 * 1024  # 5 MB
 
 
+STORAGE_EMULATOR_HOST = os.environ.get('STORAGE_EMULATOR_HOST')
+if STORAGE_EMULATOR_HOST:
+    API_ROOT = f'https://{STORAGE_EMULATOR_HOST}/storage/v1/b'
+    API_ROOT_UPLOAD = 'https://{STORAGE_EMULATOR_HOST}/upload/storage/v1/b'
+    VERIFY_SSL = False
+
+
 log = logging.getLogger(__name__)
 
 
@@ -47,10 +56,19 @@ class Storage:
     def __init__(self, *, service_file: Optional[Union[str, io.IOBase]] = None,
                  token: Optional[Token] = None,
                  session: Optional[Session] = None) -> None:
-        self.session = AioSession(session)
+        self.session = AioSession(session, verify_ssl=VERIFY_SSL)
         self.token = token or Token(service_file=service_file, scopes=SCOPES,
                                     session=self.session.session)
 
+    async def _headers(self) -> Dict[str, str]:
+        if STORAGE_EMULATOR_HOST:
+            return {}
+
+        token = await self.token.get()
+        return {
+            'Authorization': f'Bearer {token}',
+        }
+
     def get_bucket(self, bucket_name: str) -> Bucket:
         return Bucket(self, bucket_name)
 
@@ -76,8 +94,6 @@ class Storage:
 
         https://cloud.google.com/storage/docs/json_api/v1/objects/rewrite
         """
-        token = await self.token.get()
-
         if not new_name:
             new_name = object_name
 
@@ -91,8 +107,8 @@ class Storage:
         #
         # * https://cloud.google.com/storage/docs/json_api/v1/objects#resource
         headers = headers or {}
+        headers.update(await self._headers())
         headers.update({
-            'Authorization': f'Bearer {token}',
             'Content-Length': '0',
             'Content-Type': '',
         })
@@ -103,26 +119,23 @@ class Storage:
         resp = await s.post(url, headers=headers, params=params,
                             timeout=timeout)
 
-        data: dict = await resp.json()
+        data: dict = await resp.json(content_type=None)
 
         while not data.get('done') and data.get('rewriteToken'):
             params['rewriteToken'] = data['rewriteToken']
             resp = await s.post(url, headers=headers, params=params,
                                 timeout=timeout)
-            data = await resp.json()
+            data = await resp.json(content_type=None)
 
         return data
 
     async def delete(self, bucket: str, object_name: str, *,
                      params: dict = None, timeout: int = 10,
                      session: Optional[Session] = None) -> str:
-        token = await self.token.get()
         # https://cloud.google.com/storage/docs/json_api/#encoding
         encoded_object_name = quote(object_name, safe='')
         url = f'{API_ROOT}/{bucket}/o/{encoded_object_name}'
-        headers = {
-            'Authorization': f'Bearer {token}',
-        }
+        headers = await self._headers()
 
         s = AioSession(session) if session else self.session
         resp = await s.delete(url, headers=headers, params=params or {},
@@ -152,16 +165,13 @@ class Storage:
     async def list_objects(self, bucket: str, *, params: dict = None,
                            session: Optional[Session] = None,
                            timeout: int = 10) -> dict:
-        token = await self.token.get()
         url = f'{API_ROOT}/{bucket}/o'
-        headers = {
-            'Authorization': f'Bearer {token}',
-        }
+        headers = await self._headers()
 
         s = AioSession(session) if session else self.session
         resp = await s.get(url, headers=headers, params=params or {},
                            timeout=timeout)
-        data: dict = await resp.json()
+        data: dict = await resp.json(content_type=None)
         return data
 
     # TODO: if `metadata` is set, use multipart upload:
@@ -172,7 +182,6 @@ class Storage:
                      headers: dict = None, metadata: dict = None,
                      session: Optional[Session] = None, timeout: int = 30,
                      force_resumable_upload: bool = None) -> dict:
-        token = await self.token.get()
         url = f'{API_ROOT_UPLOAD}/{bucket}/o'
 
         stream = self._preprocess_data(file_data)
@@ -190,8 +199,8 @@ class Storage:
         parameters = parameters or {}
 
         headers = headers or {}
+        headers.update(await self._headers())
         headers.update({
-            'Authorization': f'Bearer {token}',
             'Content-Length': str(content_length),
             'Content-Type': content_type or '',
         })
@@ -268,13 +277,10 @@ class Storage:
     async def _download(self, bucket: str, object_name: str, *,
                         params: dict = None, timeout: int = 10,
                         session: Optional[Session] = None) -> bytes:
-        token = await self.token.get()
         # https://cloud.google.com/storage/docs/json_api/#encoding
         encoded_object_name = quote(object_name, safe='')
         url = f'{API_ROOT}/{bucket}/o/{encoded_object_name}'
-        headers = {
-            'Authorization': f'Bearer {token}',
-        }
+        headers = await self._headers()
 
         s = AioSession(session) if session else self.session
         response = await s.get(url, headers=headers, params=params or {},
@@ -297,14 +303,12 @@ class Storage:
         params['name'] = object_name
         params['uploadType'] = 'media'
 
-        headers.update({
-            'Accept': 'application/json',
-        })
+        headers.update(await self._headers())
 
         s = AioSession(session) if session else self.session
         resp = await s.post(url, data=stream, headers=headers, params=params,
                             timeout=timeout)
-        data: dict = await resp.json()
+        data: dict = await resp.json(content_type=None)
         return data
 
     async def _upload_resumable(self, url: str, object_name: str,
@@ -362,7 +366,19 @@ class Storage:
 
             break
 
-        data: dict = await resp.json()
+        data: dict = await resp.json(content_type=None)
+        return data
+
+    async def get_bucket_metadata(self, bucket: str, *, params: dict = None,
+                                  session: Optional[Session] = None,
+                                  timeout: int = 10) -> dict:
+        url = f'{API_ROOT}/{bucket}'
+        headers = await self._headers()
+
+        s = AioSession(session) if session else self.session
+        resp = await s.get(url, headers=headers, params=params or {},
+                           timeout=timeout)
+        data: dict = await resp.json(content_type=None)
         return data
 
     async def close(self):
